/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.model.sinfo;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import org.dive4elements.river.backend.SessionHolder;
import org.dive4elements.river.model.Gauge;
import org.hibernate.Query;
import org.hibernate.Session;


/**
 * Hibernate binding for the DB table daily_discharge_values
 *
 * @author Matthias Schäfer
 *
 */
@Entity
@Table(name = "daily_discharge_values")
public class DailyDischargeValue implements Serializable {

    /***** TYPES *****/

    /**
     * Field to use in a query's order-by clause
     *
     */
    public enum OrderByField {
        DAY, DISCHARGE;
    }

    /***** FIELDS *****/

    private static final long serialVersionUID = -6192738825193230784L;

    private Integer id;

    private DailyDischarge dailyDischarge;

    private Date day;

    private Double discharge;


    /***** CONSTRUCTORS *****/

    public DailyDischargeValue() {
    }

    public DailyDischargeValue(final DailyDischarge dailyDischarge, final Date day, final Double discharge) {
        this.dailyDischarge = dailyDischarge;
        this.day = day;
        this.discharge = discharge;
    }

    /**
     * Parameter constructor with primitive parameter types
     */
    public DailyDischargeValue(final DailyDischarge dailyDischarge, final Date day, final double discharge) {
        this(dailyDischarge, day, Double.valueOf(discharge));
    }


    /***** METHODS *****/

    @Id
    @SequenceGenerator(name = "SEQUENCE_DAILY_DISCHARGE_VALUE_ID_SEQ", sequenceName = "DAILY_DISCHARGE_VALUES_ID_SEQ", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQUENCE_DAILY_DISCHARGE_VALUE_ID_SEQ")
    @Column(name = "id")
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @OneToOne
    @JoinColumn(name = "daily_discharge_id")
    public DailyDischarge getDailyDischarge() {
        return this.dailyDischarge;
    }

    public void setDailyDischarge(final DailyDischarge dailyDischarge) {
        this.dailyDischarge = dailyDischarge;
    }

    @Column(name = "discharge")
    public Double getDischarge() {
        return this.discharge;
    }

    public void setDischarge(final Double discharge) {
        this.discharge = discharge;
    }

    @Column(name = "day")
    public Date getDay() {
        return this.day;
    }

    public void setDay(final Date day) {
        this.day = day;
    }

    /**
     * Selects from the database the daily discharge values of a gauge and a date range
     */
    public static List<DailyDischargeValue> getValues(final Gauge gauge, final Date startDate, final Date endDate, final OrderByField orderBy) {
        final Session session = SessionHolder.HOLDER.get();
        final String orderField = (orderBy == OrderByField.DISCHARGE) ? "discharge" : "day";
        final Query query = session.createQuery("SELECT v"
                + " FROM DailyDischargeValue AS v JOIN v.dailyDischarge AS s"
                + " WHERE (s.gauge.id=:gaugeid) AND (v.day BETWEEN :startDate AND :endDate)"
                + " ORDER BY " + orderField);
        query.setParameter("gaugeid", gauge.getId());
        query.setParameter("startDate", startDate);
        query.setParameter("endDate", endDate);
        return query.list();
    }
}
